"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetCustomField = exports.CustomFieldService = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var custom_field_model_1 = require("../models/custom-field.model");
var logger = log4js.getLogger('ServerScripts');
function CustomFieldService(app) {
    var _this = this;
    app.get('/api/customfield/get_customfields/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[CustomFieldService::get_customfields] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var disabled = req.query.disabled === 'true' || false;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetCustomFields(vcc, search_criteria, search_expression, disabled);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CustomFieldService::get_customfields] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/customfield/get_customfield/:customfield_id', function (req, res, next) {
        logger.info('[CustomFieldService::get_customfields] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var customField = {};
        var customFieldResults = [];
        var customFieldId = req.params.customfield_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!customFieldId)
                throw 'SERVER_ERROR_MISSING_PARAMETER';
            return GetCustomField(vcc, customFieldId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CustomFieldService::get_customfields] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/customfield/create_customfield', function (req, res, next) {
        logger.info('[CustomFieldService::save_customfield] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var customField = (req.body || {});
        var isCustomFieldCreated = false;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetCustomField(vcc, customField.id);
        })
            .then(function (result) {
            if (result)
                throw 'SERVER_ERROR_CUSTOM_FIELD_ID_ALREADY_EXISTS';
            return mssql_service_1.MSSql.SaveCustomField(vcc, customField.id, customField.description, customField.type, customField.label, userId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CUSTOM_FIELD_SAVE_FAILED';
            isCustomFieldCreated = true;
            var promiseArray = [];
            customField.results.map(function (r) {
                var result = new custom_field_model_1.CustomFieldResult();
                result.id = r.id;
                result.order = r.order;
                result.description = r.description;
                result.customFieldId = customField.id;
                result.createdByUserId = userId;
                promiseArray.push(mssql_service_1.MSSql.SaveCustomFieldResult(vcc, result.id, result.description, result.order, result.customFieldId, result.createdByUserId));
            });
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (result.includes(false))
                throw 'SERVER_ERROR_CUSTOM_FIELD_RESULT_SAVE_FAILED';
            if (res.headersSent)
                return;
            logger.info('[CustomFieldService::create_customfield] The CustomField save proccess has been finished.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_CUSTOM_FIELD_RESULT_SAVE_FAILED'), data: true });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CustomFieldService::create_customfield] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
            if (isCustomFieldCreated)
                _this.CustomFieldDelete(vcc, customField);
        });
    });
    app.post('/api/customfield/save_customfield', function (req, res, next) {
        logger.info('[CustomFieldService::save_customfield] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var customField = (req.body || {});
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetCustomField(vcc, customField.id);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CUSTOM_FIELD_NOT_FOUND';
            return mssql_service_1.MSSql.SaveCustomField(vcc, customField.id, customField.description, customField.type, customField.label, userId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CUSTOM_FIELD_SAVE_FAILED';
            return mssql_service_1.MSSql.DeleteCustomFieldResults(vcc, customField.id);
        })
            .then(function (result) {
            var promiseArray = [];
            customField.results.map(function (r) {
                var result = new custom_field_model_1.CustomFieldResult();
                result.id = r.id;
                result.order = r.order;
                result.description = r.description;
                result.customFieldId = customField.id;
                result.lastModifiedByUserId = userId;
                promiseArray.push(mssql_service_1.MSSql.SaveCustomFieldResult(vcc, result.id, result.description, result.order, result.customFieldId, result.lastModifiedByUserId));
            });
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (result.includes(false))
                throw 'SERVER_ERROR_CUSTOM_FIELD_RESULT_SAVE_FAILED';
            if (res.headersSent)
                return;
            logger.info('[CustomFieldService::save_customfield] The CustomField save proccess has been finished.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_CUSTOM_FIELD_RESULT_SAVE_FAILED'), data: true });
        })
            .catch(function (err) {
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            var clientMsg = (typeof err === 'string' ? err : (errorMsg || 'CANNOT_PROCESS_REQUEST'));
            logger.error('[CustomFieldService::save_customfield] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/customfield/delete_customfield', function (req, res, next) {
        logger.info('[CustomFieldService::delete_customfield] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var customField = (req.body || {});
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return CustomFieldDelete(vcc, customField);
        })
            .then(function (result) {
            if (result && result.status === false)
                throw result.error;
            if (res.headersSent)
                return;
            logger.info('[CustomFieldService::delete_customfield] The CustomField delete proccess has been finished.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CustomFieldService::delete_customfield] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    function CustomFieldDelete(vcc, customField) {
        var genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.error('[CustomFieldService::CustomFieldDelete] Result: ' + result);
            return mssql_service_1.MSSql.DeleteCustomField(vcc, customField.id);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CUSTOM_FIELD_DELETE_FAILED';
            genericResult.status = true;
            genericResult.description = 'OK';
            return Promise.resolve(genericResult);
        })
            .catch(function (err) {
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_CUSTOM_FIELD_DELETE_FAILED');
            logger.error('[CustomFieldService::CustomFieldDelete] Error: ' + errorMsg);
            genericResult.status = false;
            genericResult.description = 'SERVER_ERROR_UNKNOWN_MESSAGE';
            genericResult.error = errorMsg;
            return Promise.resolve(genericResult);
        });
    }
    ;
}
exports.CustomFieldService = CustomFieldService;
function GetCustomField(vcc, customFieldId, evaluationPlanId) {
    if (evaluationPlanId === void 0) { evaluationPlanId = null; }
    var customField = {};
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        return mssql_service_1.MSSql.GetCustomField(vcc, customFieldId, evaluationPlanId);
    })
        .then(function (result) {
        customField = result;
        if (customField)
            return mssql_service_1.MSSql.GetCustomFieldResults(vcc, customFieldId, evaluationPlanId);
        else
            return Promise.resolve(true);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_CUSTOM_FIELD_RESULTS_NOT_FOUND';
        if (customField)
            customField.results = result;
        return Promise.resolve(customField);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[CustomFieldService::GetCustomField] Error: ' + errorMsg);
        return Promise.reject(false);
    });
}
exports.GetCustomField = GetCustomField;
